@@ -1,14 +1,12 @@
-use std::{collections::HashMap, net::IpAddr, os::fd::BorrowedFd, sync::Once};
-
-use ipnet::IpNet;
-use log::{debug, error};
-use netlink_packet_route::link::{
-    BridgeVlanInfoFlags, InfoBridge, InfoData, InfoKind, InfoVeth, LinkAttribute, LinkInfo,
-    LinkMessage,
-};
+use std::{collections::HashMap, fs, net::IpAddr, os::fd::BorrowedFd};
 
 use crate::dns::aardvark::SafeString;
+use crate::network::core_utils::get_default_route_interface;
 use crate::network::dhcp::{dhcp_teardown, get_dhcp_lease};
+use crate::network::netlink::Socket;
+use crate::network::netlink_route::{
+    parse_create_link_options, CreateLinkOptions, LinkID, NetlinkRoute,
+};
 use crate::{
     dns::aardvark::AardvarkEntry,
     error::{ErrorWrap, NetavarkError, NetavarkErrorList, NetavarkResult},
@@ -17,8 +15,16 @@ use crate::{
         iptables::MAX_HASH_SIZE,
         state::{remove_fw_config, write_fw_config},
     },
-    network::{constants, core_utils::disable_ipv6_autoconf, types},
+    network::{constants, sysctl::disable_ipv6_autoconf, types},
 };
+use ipnet::IpNet;
+use log::{debug, error};
+use netlink_packet_route::address::{AddressAttribute, AddressScope};
+use netlink_packet_route::link::{
+    BridgeVlanInfoFlags, InfoBridge, InfoData, InfoKind, InfoVeth, LinkAttribute, LinkInfo,
+    LinkMessage,
+};
+use netlink_packet_route::AddressFamily;
 
 use super::{
     constants::{
@@ -26,13 +32,13 @@ use super::{
         NO_CONTAINER_INTERFACE_ERROR, OPTION_HOST_INTERFACE_NAME, OPTION_ISOLATE, OPTION_METRIC,
         OPTION_MODE, OPTION_MTU, OPTION_NO_DEFAULT_ROUTE, OPTION_VLAN, OPTION_VRF,
     },
-    core_utils::{self, get_ipam_addresses, join_netns, parse_option, CoreUtils},
+    core_utils::{self, get_ipam_addresses, is_using_systemd, join_netns, parse_option, CoreUtils},
     driver::{self, DriverInfo},
     internal_types::{
         IPAMAddresses, IsolateOption, PortForwardConfig, SetupNetwork, TearDownNetwork,
         TeardownPortForward,
     },
-    netlink,
+    sysctl,
     types::StatusBlock,
 };
 
@@ -78,6 +84,16 @@ pub struct Bridge<'a> {
     data: Option<InternalData>,
 }
 
+struct CreateInterfacesResult {
+    /// The MAC address of the container's veth interface.
+    mac_address: String,
+    /// An optional writer for creating a sysctl.d config file.
+    /// This is only created when a new bridge is created.
+    sysctl_writer: Option<sysctl::SysctlDWriter<'static, String, String>>,
+    /// The interface index of the bridge.
+    bridge_index: u32,
+}
+
 impl<'a> Bridge<'a> {
     pub fn new(info: DriverInfo<'a>) -> Self {
         Bridge { info, data: None }
@@ -146,8 +162,8 @@ impl driver::NetworkDriver for Bridge<'_> {
 
     fn setup(
         &self,
-        netlink_sockets: (&mut netlink::Socket, &mut netlink::Socket),
-    ) -> NetavarkResult<(StatusBlock, Option<AardvarkEntry>)> {
+        netlink_sockets: (&mut Socket<NetlinkRoute>, &mut Socket<NetlinkRoute>),
+    ) -> NetavarkResult<(StatusBlock, Option<AardvarkEntry<'_>>)> {
         let data = match &self.data {
             Some(d) => d,
             None => return Err(NetavarkError::msg("must call validate() before setup()")),
@@ -163,22 +179,18 @@ impl driver::NetworkDriver for Bridge<'_> {
             data.bridge_interface_name, data.ipam.gateway_addresses
         );
 
-        if let BridgeMode::Managed = data.mode {
-            if !self.info.network.internal {
-                setup_ipv4_fw_sysctl()?;
-                if data.ipam.ipv6_enabled {
-                    setup_ipv6_fw_sysctl()?;
-                }
-            }
-        }
-
         let (host_sock, netns_sock) = netlink_sockets;
 
-        let container_veth_mac = create_interfaces(
+        let CreateInterfacesResult {
+            mac_address: container_veth_mac,
+            sysctl_writer,
+            bridge_index,
+        } = create_interfaces(
             host_sock,
             netns_sock,
             data,
             self.info.network.internal,
+            self.info.rootless,
             self.info.netns_host,
             self.info.netns_container,
         )?;
@@ -259,18 +271,49 @@ impl driver::NetworkDriver for Bridge<'_> {
                 }
             }
 
-            let gw = data
-                .ipam
-                .gateway_addresses
-                .iter()
-                .map(|ipnet| ipnet.addr())
-                .collect();
+            // Fixes #1177: In unmanaged mode, the gateway IP may not be on the host.
+            // We need to find an IP on the bridge itself for aardvark-dns to bind to.
+            let bind_addr: Vec<IpAddr> = if data.mode == BridgeMode::Unmanaged {
+                let addr_msgs = host_sock.dump_addresses(Some(bridge_index))?;
+
+                let addresses: Vec<IpAddr> = addr_msgs
+                    .into_iter()
+                    .filter_map(|addr_msg| {
+                        // address is either a IPv4 address, or it's an IPv6 address that is not a link-local address.
+                        if (addr_msg.header.family == AddressFamily::Inet6
+                            && addr_msg.header.scope != AddressScope::Link)
+                            || addr_msg.header.family == AddressFamily::Inet
+                        {
+                            addr_msg.attributes.into_iter().find_map(|attr| {
+                                if let AddressAttribute::Address(ip) = attr {
+                                    Some(ip)
+                                } else {
+                                    None
+                                }
+                            })
+                        } else {
+                            None
+                        }
+                    })
+                    .collect();
+                if addresses.is_empty() {
+                    return Err(NetavarkError::msg(format!("bridge '{}' in unmanaged mode has no usable IP addresses. Aardvark-dns requires at least one address (should not be an IPv6 link-local address) to bind to. Please add an IP address or disable DNS for this network (--disable-dns).", data.bridge_interface_name)));
+                }
+                response.dns_server_ips = Some(addresses.clone());
+                addresses
+            } else {
+                data.ipam
+                    .gateway_addresses
+                    .iter()
+                    .map(|ipnet| ipnet.addr())
+                    .collect()
+            };
 
             match self.info.container_id.as_str().try_into() {
                 Ok(id) => Some(AardvarkEntry {
                     network_name: &self.info.network.name,
                     container_id: id,
-                    network_gateways: gw,
+                    network_gateways: bind_addr,
                     network_dns_servers: &self.info.network.network_dns_servers,
                     container_ips_v4: ipv4,
                     container_ips_v6: ipv6,
@@ -295,35 +338,21 @@ impl driver::NetworkDriver for Bridge<'_> {
         };
 
         if let BridgeMode::Managed = data.mode {
-            // if the network is internal block routing and do not setup firewall rules
-            if self.info.network.internal {
-                CoreUtils::apply_sysctl_value(
-                    format!(
-                        "/proc/sys/net/ipv4/conf/{}/forwarding",
-                        data.bridge_interface_name
-                    ),
-                    "0",
-                )?;
-                if data.ipam.ipv6_enabled {
-                    CoreUtils::apply_sysctl_value(
-                        format!(
-                            "/proc/sys/net/ipv6/conf/{}/forwarding",
-                            data.bridge_interface_name
-                        ),
-                        "0",
-                    )?;
-                }
-            } else {
+            // if the network is internal do not setup firewall rules
+            if !self.info.network.internal {
                 self.setup_firewall(data)?
             }
         }
+        if let Some(w) = sysctl_writer {
+            w.commit();
+        }
 
         Ok((response, aardvark_entry))
     }
 
     fn teardown(
         &self,
-        netlink_sockets: (&mut netlink::Socket, &mut netlink::Socket),
+        netlink_sockets: (&mut Socket<NetlinkRoute>, &mut Socket<NetlinkRoute>),
     ) -> NetavarkResult<()> {
         let mode: Option<String> = parse_option(&self.info.network.options, OPTION_MODE)?;
         let mode = get_bridge_mode_from_string(mode.as_deref())?;
@@ -357,6 +386,19 @@ impl driver::NetworkDriver for Bridge<'_> {
         };
 
         if !self.info.network.internal && mode == BridgeMode::Managed {
+            if complete_teardown {
+                // delete sysctl file as well
+                let path = sysctl::get_bridge_sysctl_d_path(&bridge_name);
+                if let Err(e) = fs::remove_file(&path) {
+                    if e.kind() != std::io::ErrorKind::NotFound {
+                        error_list.push(NetavarkError::wrap(
+                            format!("failed to remove {path}"),
+                            e.into(),
+                        ));
+                    }
+                };
+            }
+
             match self.teardown_firewall(complete_teardown, bridge_name) {
                 Ok(_) => {}
                 Err(err) => {
@@ -476,19 +518,6 @@ impl<'a> Bridge<'a> {
 
         self.info.firewall.setup_network(sn, &system_dbus)?;
 
-        if spf.port_mappings.is_some() {
-            // Need to enable sysctl localnet so that traffic can pass
-            // through localhost to containers
-
-            CoreUtils::apply_sysctl_value(
-                format!(
-                    "net.ipv4.conf.{}.route_localnet",
-                    data.bridge_interface_name
-                ),
-                "1",
-            )?;
-        }
-
         self.info.firewall.setup_port_forward(spf, &system_dbus)?;
         Ok(())
     }
@@ -507,7 +536,7 @@ impl<'a> Bridge<'a> {
             None => {
                 let isolate = get_isolate_option(&self.info.network.options).unwrap_or_else(|e| {
                     // just log we still try to do as much as possible for cleanup
-                    error!("failed to parse {} option: {}", OPTION_ISOLATE, e);
+                    error!("failed to parse {OPTION_ISOLATE} option: {e}");
                     IsolateOption::Never
                 });
 
@@ -516,7 +545,7 @@ impl<'a> Bridge<'a> {
                         Ok(i) => (i.container_addresses, i.nameservers),
                         Err(e) => {
                             // just log we still try to do as much as possible for cleanup
-                            error!("failed to parse ipam options: {}", e);
+                            error!("failed to parse ipam options: {e}");
                             (Vec::new(), Vec::new())
                         }
                     };
@@ -562,62 +591,36 @@ impl<'a> Bridge<'a> {
 }
 
 // sysctl forward
-
-static IPV4_FORWARD_ONCE: Once = Once::new();
-static IPV6_FORWARD_ONCE: Once = Once::new();
-
-const IPV4_FORWARD: &str = "net.ipv4.ip_forward";
-const IPV6_FORWARD: &str = "net.ipv6.conf.all.forwarding";
-
-fn setup_ipv4_fw_sysctl() -> NetavarkResult<()> {
-    let mut result = Ok("".to_string());
-
-    IPV4_FORWARD_ONCE.call_once(|| {
-        result = CoreUtils::apply_sysctl_value(IPV4_FORWARD, "1");
-    });
-
-    match result {
-        Ok(_) => {}
-        Err(e) => return Err(e.into()),
-    };
-    Ok(())
-}
-
-fn setup_ipv6_fw_sysctl() -> NetavarkResult<()> {
-    let mut result = Ok("".to_string());
-
-    IPV6_FORWARD_ONCE.call_once(|| {
-        result = CoreUtils::apply_sysctl_value(IPV6_FORWARD, "1");
-    });
-
-    match result {
-        Ok(_) => {}
-        Err(e) => return Err(e.into()),
-    };
-    Ok(())
-}
+const IPV4_FORWARD: &str = "net/ipv4/ip_forward";
+const IPV6_FORWARD: &str = "net/ipv6/conf/all/forwarding";
 
 /// returns the container veth mac address
 fn create_interfaces(
-    host: &mut netlink::Socket,
-    netns: &mut netlink::Socket,
+    host: &mut Socket<NetlinkRoute>,
+    netns: &mut Socket<NetlinkRoute>,
     data: &InternalData,
     internal: bool,
+    rootless: bool,
     hostns_fd: BorrowedFd<'_>,
     netns_fd: BorrowedFd<'_>,
-) -> NetavarkResult<String> {
-    let (bridge_index, mac) = match host.get_link(netlink::LinkID::Name(
-        data.bridge_interface_name.to_string(),
-    )) {
-        Ok(bridge) => (
-            validate_bridge_link(
+) -> NetavarkResult<CreateInterfacesResult> {
+    let mut sysctl_writer = None;
+    let (bridge_index, mtu, mac) = match host
+        .get_link(LinkID::Name(data.bridge_interface_name.to_string()))
+    {
+        Ok(bridge) => {
+            let (bridge_index, mtu) = validate_bridge_link(
                 bridge,
                 data.vlan.is_some(),
                 host,
                 &data.bridge_interface_name,
-            )?,
-            None,
-        ),
+            )?;
+            (
+                bridge_index,
+                if data.mtu == 0 { mtu } else { data.mtu },
+                None,
+            )
+        }
         Err(err) => match err.unwrap() {
             NetavarkError::Netlink(e) => {
                 if -e.raw_code() != libc::ENODEV {
@@ -631,10 +634,92 @@ fn create_interfaces(
                         .wrap("in unmanaged mode, the bridge must already exist on the host");
                 }
 
-                let mut create_link_opts = netlink::CreateLinkOptions::new(
+                // Create sysctl writer, when using systemd and running as root it is possible that the
+                // global configured sysctl.d config conflict with the values we write. In such cases
+                // systemd-sysctl might overright the value causing hard to find errors:
+                // https://issues.redhat.com/browse/RHEL-89477
+                // This writer is used to generate a matching sysctl.d config file for our values so
+                // systemd-sysctl is aware of them.
+                let path = if is_using_systemd() && !rootless {
+                    let _ = fs::create_dir("/run/sysctl.d");
+                    Some(sysctl::get_bridge_sysctl_d_path(
+                        &data.bridge_interface_name,
+                    ))
+                } else {
+                    None
+                };
+
+                let mut sysctls = Vec::with_capacity(6);
+
+                // if internal block routing on the bridge otherwise enable routing globally
+                if internal {
+                    sysctls.push((
+                        format!("net/ipv4/conf/{}/forwarding", data.bridge_interface_name),
+                        "0",
+                    ));
+                    if data.ipam.ipv6_enabled {
+                        sysctls.push((
+                            format!("net/ipv6/conf/{}/forwarding", data.bridge_interface_name),
+                            "0",
+                        ));
+                    }
+                } else {
+                    sysctls.push((IPV4_FORWARD.to_string(), "1"));
+                    if data.ipam.ipv6_enabled {
+                        sysctls.push((IPV6_FORWARD.to_string(), "1"));
+                    }
+                    sysctls.push((
+                        format!(
+                            "net/ipv4/conf/{}/route_localnet",
+                            data.bridge_interface_name
+                        ),
+                        "1",
+                    ));
+                }
+
+                if data.ipam.ipv6_enabled {
+                    // Disable duplicate address detection if ipv6 enabled
+                    // Do not accept Router Advertisements if ipv6 is enabled
+                    let br_accept_dad =
+                        format!("net/ipv6/conf/{}/accept_dad", &data.bridge_interface_name);
+                    let br_accept_ra =
+                        format!("net/ipv6/conf/{}/accept_ra", &data.bridge_interface_name);
+                    sysctls.push((br_accept_dad, "0"));
+                    sysctls.push((br_accept_ra, "0"));
+                }
+
+                // Disable strict reverse path search validation. On RHEL it is set to strict mode
+                // which breaks port forwarding when multiple networks are attached as the package
+                // may be routed over a different interface on the reverse path.
+                // As documented for the sysctl for complicated or asymmetric routing loose mode (2)
+                // is recommended.
+                let br_rp_filter =
+                    format!("net/ipv4/conf/{}/rp_filter", &data.bridge_interface_name);
+                sysctls.push((br_rp_filter, "2"));
+
+                // writer must be create before the bridge is created
+                let sw = sysctl::SysctlDWriter::new(path, sysctls);
+
+                let mut create_link_opts = CreateLinkOptions::new(
                     data.bridge_interface_name.to_string(),
                     InfoKind::Bridge,
                 );
+
+                let mut mtu = data.mtu;
+                if mtu == 0 {
+                    // if we have a default route, use its mtu as default
+                    if let Ok(link) = get_default_route_interface(host) {
+                        match core_utils::get_mtu_from_iface_attributes(&link.attributes) {
+                            Ok(iface_mtu) => {
+                                debug!("Using mtu {iface_mtu} from default route interface for the network");
+                                mtu = iface_mtu;
+                            },
+                            Err(e) => log::warn!(
+                                "failed to get mtu for default interface {}: {e}, using kernel default", link.header.index
+                            ),
+                        }
+                    }
+                }
                 create_link_opts.mtu = data.mtu;
 
                 if data.vlan.is_some() {
@@ -643,7 +728,7 @@ fn create_interfaces(
                 }
 
                 if let Some(vrf_name) = &data.vrf {
-                    let vrf = match host.get_link(netlink::LinkID::Name(vrf_name.to_string())) {
+                    let vrf = match host.get_link(LinkID::Name(vrf_name.to_string())) {
                         Ok(vrf) => check_link_is_vrf(vrf, vrf_name)?,
                         Err(err) => return Err(err).wrap("get vrf to set up bridge interface"),
                     };
@@ -652,40 +737,19 @@ fn create_interfaces(
 
                 host.create_link(create_link_opts).wrap("create bridge")?;
 
-                if data.ipam.ipv6_enabled {
-                    // Disable duplicate address detection if ipv6 enabled
-                    // Do not accept Router Advertisements if ipv6 is enabled
-                    let br_accept_dad = format!(
-                        "/proc/sys/net/ipv6/conf/{}/accept_dad",
-                        &data.bridge_interface_name
-                    );
-                    let br_accept_ra =
-                        format!("net/ipv6/conf/{}/accept_ra", &data.bridge_interface_name);
-                    CoreUtils::apply_sysctl_value(br_accept_dad, "0")?;
-                    CoreUtils::apply_sysctl_value(br_accept_ra, "0")?;
-                }
-
-                // Disable strict reverse path search validation. On RHEL it is set to strict mode
-                // which breaks port forwarding when multiple networks are attached as the package
-                // may be routed over a different interface on the reverse path.
-                // As documented for the sysctl for complicated or asymmetric routing loose mode (2)
-                // is recommended.
-                let br_rp_filter = format!(
-                    "/proc/sys/net/ipv4/conf/{}/rp_filter",
-                    &data.bridge_interface_name
-                );
-                CoreUtils::apply_sysctl_value(br_rp_filter, "2")?;
+                // Note sysctls must be written after the bridge is created
+                sw.write_sysctls()?;
+                sysctl_writer = Some(sw);
 
                 let link = host
-                    .get_link(netlink::LinkID::Name(
-                        data.bridge_interface_name.to_string(),
-                    ))
+                    .get_link(LinkID::Name(data.bridge_interface_name.to_string()))
                     .wrap("get bridge interface")?;
 
                 let mut mac = None;
                 for nla in link.attributes.into_iter() {
                     if let LinkAttribute::Address(addr) = nla {
                         mac = Some(addr);
+                        break;
                     }
                 }
                 if mac.is_none() {
@@ -699,16 +763,16 @@ fn create_interfaces(
                         .wrap("add ip addr to bridge")?;
                 }
 
-                host.set_up(netlink::LinkID::ID(link.header.index))
+                host.set_up(LinkID::ID(link.header.index))
                     .wrap("set bridge up")?;
 
-                (link.header.index, mac)
+                (link.header.index, mtu, mac)
             }
             _ => return Err(err),
         },
     };
 
-    create_veth_pair(
+    let mac = create_veth_pair(
         host,
         netns,
         data,
@@ -717,33 +781,39 @@ fn create_interfaces(
         internal,
         hostns_fd,
         netns_fd,
-    )
+        mtu,
+    )?;
+    Ok(CreateInterfacesResult {
+        mac_address: mac,
+        sysctl_writer,
+        bridge_index,
+    })
 }
 
 /// return the container veth mac address
 #[allow(clippy::too_many_arguments)]
 fn create_veth_pair<'fd>(
-    host: &mut netlink::Socket,
-    netns: &mut netlink::Socket,
+    host: &mut Socket<NetlinkRoute>,
+    netns: &mut Socket<NetlinkRoute>,
     data: &InternalData,
     primary_index: u32,
     bridge_mac: Option<Vec<u8>>,
     internal: bool,
     hostns_fd: BorrowedFd<'fd>,
     netns_fd: BorrowedFd<'fd>,
+    mtu: u32,
 ) -> NetavarkResult<String> {
     let mut peer_opts =
-        netlink::CreateLinkOptions::new(data.container_interface_name.to_string(), InfoKind::Veth);
+        CreateLinkOptions::new(data.container_interface_name.to_string(), InfoKind::Veth);
     peer_opts.mac = data.mac_address.clone().unwrap_or_default();
-    peer_opts.mtu = data.mtu;
+    peer_opts.mtu = mtu;
     peer_opts.netns = Some(netns_fd);
 
     let mut peer = LinkMessage::default();
-    netlink::parse_create_link_options(&mut peer, peer_opts);
+    parse_create_link_options(&mut peer, peer_opts);
 
-    let mut host_veth =
-        netlink::CreateLinkOptions::new(data.host_interface_name.clone(), InfoKind::Veth);
-    host_veth.mtu = data.mtu;
+    let mut host_veth = CreateLinkOptions::new(data.host_interface_name.clone(), InfoKind::Veth);
+    host_veth.mtu = mtu;
     host_veth.primary_index = primary_index;
     host_veth.info_data = Some(InfoData::Veth(InfoVeth::Peer(peer)));
 
@@ -766,9 +836,7 @@ fn create_veth_pair<'fd>(
     })?;
 
     let veth = netns
-        .get_link(netlink::LinkID::Name(
-            data.container_interface_name.to_string(),
-        ))
+        .get_link(LinkID::Name(data.container_interface_name.to_string()))
         .wrap("get container veth")?;
 
     let mut mac = String::from("");
@@ -798,48 +866,43 @@ fn create_veth_pair<'fd>(
     }
 
     if let BridgeMode::Managed = data.mode {
-        exec_netns!(hostns_fd, netns_fd, res, {
+        exec_netns!(hostns_fd, netns_fd, {
             disable_ipv6_autoconf(&data.container_interface_name)?;
             if data.ipam.ipv6_enabled {
                 //  Disable dad inside the container too
                 let disable_dad_in_container = format!(
-                    "/proc/sys/net/ipv6/conf/{}/accept_dad",
+                    "net/ipv6/conf/{}/accept_dad",
                     &data.container_interface_name
                 );
-                core_utils::CoreUtils::apply_sysctl_value(disable_dad_in_container, "0")?;
+                sysctl::apply_sysctl_value(disable_dad_in_container, "0")?;
             }
             let enable_arp_notify = format!(
-                "/proc/sys/net/ipv4/conf/{}/arp_notify",
+                "net/ipv4/conf/{}/arp_notify",
                 &data.container_interface_name
             );
-            core_utils::CoreUtils::apply_sysctl_value(enable_arp_notify, "1")?;
+            sysctl::apply_sysctl_value(enable_arp_notify, "1")?;
 
             // disable strict reverse path search validation
-            let rp_filter = format!(
-                "/proc/sys/net/ipv4/conf/{}/rp_filter",
-                &data.container_interface_name
-            );
-            CoreUtils::apply_sysctl_value(rp_filter, "2")?;
+            let rp_filter = format!("net/ipv4/conf/{}/rp_filter", &data.container_interface_name);
+            sysctl::apply_sysctl_value(rp_filter, "2")?;
             Ok::<(), NetavarkError>(())
-        });
-        // check the result and return error
-        res?;
+        })?;
 
         if data.ipam.ipv6_enabled {
-            let host_veth = host.get_link(netlink::LinkID::ID(host_link))?;
+            let host_veth = host.get_link(LinkID::ID(host_link))?;
 
             for nla in host_veth.attributes.into_iter() {
                 if let LinkAttribute::IfName(name) = nla {
                     //  Disable dad inside on the host too
-                    let disable_dad_in_container =
-                        format!("/proc/sys/net/ipv6/conf/{name}/accept_dad");
-                    core_utils::CoreUtils::apply_sysctl_value(disable_dad_in_container, "0")?;
+                    let disable_dad_in_container = format!("net/ipv6/conf/{name}/accept_dad");
+                    sysctl::apply_sysctl_value(disable_dad_in_container, "0")?;
+                    break;
                 }
             }
         }
     }
 
-    host.set_up(netlink::LinkID::ID(host_link))
+    host.set_up(LinkID::ID(host_link))
         .wrap("failed to set host veth up")?;
 
     // Ok this is extremely strange, by default the kernel will always choose the mac address with the
@@ -852,7 +915,7 @@ fn create_veth_pair<'fd>(
     // connected otherwise no connectivity is possible at all and I have no idea why but CNI does it
     // also in the same way.
     if let Some(m) = bridge_mac {
-        host.set_mac_address(netlink::LinkID::ID(primary_index), m)
+        host.set_mac_address(LinkID::ID(primary_index), m)
             .wrap("set static mac on bridge")?;
     }
 
@@ -863,7 +926,7 @@ fn create_veth_pair<'fd>(
     }
 
     netns
-        .set_up(netlink::LinkID::ID(veth.header.index))
+        .set_up(LinkID::ID(veth.header.index))
         .wrap("set container veth up")?;
 
     if !internal && !data.no_default_route {
@@ -884,10 +947,15 @@ fn create_veth_pair<'fd>(
 fn validate_bridge_link(
     msg: LinkMessage,
     vlan: bool,
-    netlink: &mut netlink::Socket,
+    netlink: &mut Socket<NetlinkRoute>,
     br_name: &str,
-) -> NetavarkResult<u32> {
+) -> NetavarkResult<(u32, u32)> {
+    let mut mtu: u32 = 0;
+    let mut header_index: u32 = 0;
     for nla in msg.attributes.iter() {
+        if let LinkAttribute::Mtu(m) = nla {
+            mtu = *m;
+        }
         if let LinkAttribute::LinkInfo(info) = nla {
             // when vlan is requested also check the VlanFiltering attribute
             if vlan {
@@ -925,7 +993,8 @@ fn validate_bridge_link(
             for inf in info.iter() {
                 if let LinkInfo::Kind(kind) = inf {
                     if *kind == InfoKind::Bridge {
-                        return Ok(msg.header.index);
+                        header_index = msg.header.index;
+                        break;
                     } else {
                         return Err(NetavarkError::Message(format!(
                             "bridge interface {br_name} already exists but is a {kind:?} interface"
@@ -935,6 +1004,11 @@ fn validate_bridge_link(
             }
         }
     }
+
+    if header_index != 0 {
+        return Ok((header_index, mtu));
+    }
+
     Err(NetavarkError::Message(format!(
         "could not determine namespace link kind for bridge {br_name}"
     )))
@@ -950,8 +1024,7 @@ fn check_link_is_vrf(msg: LinkMessage, vrf_name: &str) -> NetavarkResult<LinkMes
                         return Ok(msg);
                     } else {
                         return Err(NetavarkError::Message(format!(
-                            "vrf {} already exists but is a {:?} interface",
-                            vrf_name, kind
+                            "vrf {vrf_name} already exists but is a {kind:?} interface"
                         )));
                     }
                 }
@@ -959,26 +1032,25 @@ fn check_link_is_vrf(msg: LinkMessage, vrf_name: &str) -> NetavarkResult<LinkMes
         }
     }
     Err(NetavarkError::Message(format!(
-        "could not determine namespace link kind for vrf {}",
-        vrf_name
+        "could not determine namespace link kind for vrf {vrf_name}"
     )))
 }
 
 fn remove_link(
-    host: &mut netlink::Socket,
-    netns: &mut netlink::Socket,
+    host: &mut Socket<NetlinkRoute>,
+    netns: &mut Socket<NetlinkRoute>,
     mode: BridgeMode,
     br_name: &str,
     container_veth_name: &str,
 ) -> NetavarkResult<bool> {
     netns
-        .del_link(netlink::LinkID::Name(container_veth_name.to_string()))
+        .del_link(LinkID::Name(container_veth_name.to_string()))
         .wrap(format!(
             "failed to delete container veth {container_veth_name}"
         ))?;
 
     let br = host
-        .get_link(netlink::LinkID::Name(br_name.to_string()))
+        .get_link(LinkID::Name(br_name.to_string()))
         .wrap("failed to get bridge interface")?;
 
     let links = host
@@ -987,8 +1059,8 @@ fn remove_link(
     // no connected interfaces on that bridge we can remove it
     if links.is_empty() {
         if let BridgeMode::Managed = mode {
-            log::info!("removing bridge {}", br_name);
-            host.del_link(netlink::LinkID::ID(br.header.index))
+            log::info!("removing bridge {br_name}");
+            host.del_link(LinkID::ID(br.header.index))
                 .wrap(format!("failed to delete bridge {container_veth_name}"))?;
             return Ok(true);
         }
@@ -1021,9 +1093,6 @@ fn get_bridge_mode_from_string(mode: Option<&str>) -> NetavarkResult<BridgeMode>
 fn maybe_add_alias<'a>(names: &mut Vec<SafeString<'a>>, name: &'a str) {
     match name.try_into() {
         Ok(name) => names.push(name),
-        Err(err) => log::warn!(
-            "invalid network alias {:?}: {err}, ignoring this name",
-            name
-        ),
+        Err(err) => log::warn!("invalid network alias {name:?}: {err}, ignoring this name"),
     }
 }
